import logging
import os

from bravado.exception import HTTPNotFound, HTTPUnprocessableEntity, HTTPError
from jsonschema.exceptions import RefResolutionError

from django.conf import settings
from esi.clients import esi_client_factory

from allianceauth import __version__
from allianceauth.utils.django import StartupCommand


SWAGGER_SPEC_PATH = os.path.join(os.path.dirname(
    os.path.abspath(__file__)), 'swagger.json'
)

# for the love of Bob please add operations you use here. I'm tired of breaking undocumented things.
ESI_OPERATIONS=[
    'get_alliances_alliance_id',
    'get_alliances_alliance_id_corporations',
    'get_corporations_corporation_id',
    'get_characters_character_id',
    'post_characters_affiliation',
    'get_universe_types_type_id',
    'get_universe_factions',
    'post_universe_names',
]


logger = logging.getLogger(__name__)


class ObjectNotFound(Exception):
    def __init__(self, obj_id, type_name):
        self.id = obj_id
        self.type = type_name

    def __str__(self):
        return f'{self.type} with ID {self.id} not found.'


class Entity:
    def __init__(self, id=None, name=None, **kwargs):
        super().__init__(**kwargs)
        self.id = id
        self.name = name

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"<{self.__class__.__name__} ({self.id}): {self.name}>"

    def __bool__(self):
        return bool(self.id)

    def __eq__(self, other):
        return self.id == other.id


class AllianceMixin:
    def __init__(self, alliance_id=None, **kwargs):
        super().__init__(**kwargs)
        self.alliance_id = alliance_id
        self._alliance = None

    @property
    def alliance(self):
        if self.alliance_id:
            if not self._alliance:
                self._alliance = provider.get_alliance(self.alliance_id)
            return self._alliance
        return Entity(None, None)


class FactionMixin:
    def __init__(self, faction_id=None, **kwargs):
        super().__init__(**kwargs)
        self.faction_id = faction_id
        self._faction = None

    @property
    def faction(self):
        if self.faction_id:
            if not self._faction:
                self._faction = provider.get_faction(self.faction_id)
            return self._faction
        return Entity(None, None)


class Corporation(Entity, AllianceMixin, FactionMixin):
    def __init__(self, ticker=None, ceo_id=None, members=None, **kwargs):
        super().__init__(**kwargs)
        self.ticker = ticker
        self.ceo_id = ceo_id
        self.members = members
        self._ceo = None

    @property
    def ceo(self):
        if not self._ceo:
            self._ceo = provider.get_character(self.ceo_id)
        return self._ceo


class Alliance(Entity, FactionMixin):
    def __init__(self, ticker=None, corp_ids=None, executor_corp_id=None, **kwargs):
        super().__init__(**kwargs)
        self.ticker = ticker
        self.corp_ids = corp_ids
        self.executor_corp_id = executor_corp_id
        self._corps = {}

    def corp(self, id):
        assert id in self.corp_ids
        if id not in self._corps:
            self._corps[id] = provider.get_corp(id)
            self._corps[id]._alliance = self
        return self._corps[id]

    @property
    def corps(self):
        return sorted((self.corp(c_id) for c_id in self.corp_ids), key=lambda x: x.name)

    @property
    def executor_corp(self):
        if self.executor_corp_id:
            return self.corp(self.executor_corp_id)
        return Entity(None, None)


class Character(Entity, AllianceMixin, FactionMixin):
    def __init__(self, corp_id=None, **kwargs):
        super().__init__(**kwargs)
        self.corp_id = corp_id
        self._corp = None

    @property
    def corp(self):
        if not self._corp:
            self._corp = provider.get_corp(self.corp_id)
        return self._corp


class ItemType(Entity):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class EveProvider:
    def get_alliance(self, alliance_id):
        """
        :return: an Alliance object for the given ID
        """
        raise NotImplementedError()

    def get_corp(self, corp_id):
        """
        :return: a Corporation object for the given ID
        """
        raise NotImplementedError()

    def get_character(self, character_id):
        """
        :return: a Character object for the given ID
        """
        raise NotImplementedError()

    def get_itemtype(self, type_id):
        """
        :return: an ItemType object for the given ID
        """
        raise NotImplementedError()


class EveSwaggerProvider(EveProvider):
    def __init__(self, token=None, adapter=None):
        if settings.DEBUG or StartupCommand().is_management_command:
            self._client = None
            logger.info('ESI client will be loaded on-demand')
        else:
            logger.info('Loading ESI client')
            try:
                self._client = esi_client_factory(
                    token=token,
                    spec_file=SWAGGER_SPEC_PATH,
                    app_info_text=f"allianceauth v{__version__}"
                )
            except (HTTPError, RefResolutionError):
                logger.exception(
                    'Failed to load ESI client on startup. '
                    'Switching to on-demand loading for ESI client.'
                )
                self._client = None

        self._token = token
        self.adapter = adapter or self
        self._faction_list = None  # what are the odds this will change? could cache forever!

    @property
    def client(self):
        if self._client is None:
            self._client = esi_client_factory(
                token=self._token, spec_file=SWAGGER_SPEC_PATH, app_info_text=("allianceauth v" + __version__)
            )
        return self._client

    def __str__(self):
        return 'esi'

    def get_alliance(self, alliance_id: int) -> Alliance:
        """Fetch alliance from ESI."""
        try:
            data = self.client.Alliance.get_alliances_alliance_id(alliance_id=alliance_id).result()
            corps = self.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance_id).result()
            model = Alliance(
                id=alliance_id,
                name=data['name'],
                ticker=data['ticker'],
                corp_ids=corps,
                executor_corp_id=data['executor_corporation_id'] if 'executor_corporation_id' in data else None,
                faction_id=data['faction_id'] if 'faction_id' in data else None,
            )
            return model
        except HTTPNotFound:
            raise ObjectNotFound(alliance_id, 'alliance')

    def get_corp(self, corp_id: int) -> Corporation:
        """Fetch corporation from ESI."""
        try:
            data = self.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).result()
            model = Corporation(
                id=corp_id,
                name=data['name'],
                ticker=data['ticker'],
                ceo_id=data['ceo_id'],
                members=data['member_count'],
                alliance_id=data['alliance_id'] if 'alliance_id' in data else None,
                faction_id=data['faction_id'] if 'faction_id' in data else None,
            )
            return model
        except HTTPNotFound:
            raise ObjectNotFound(corp_id, 'corporation')

    def get_character(self, character_id: int) -> Character:
        """Fetch character from ESI."""
        try:
            character_name = self._fetch_character_name(character_id)
            affiliation = self.client.Character.post_characters_affiliation(characters=[character_id]).result()[0]
            model = Character(
                id=character_id,
                name=character_name,
                corp_id=affiliation['corporation_id'],
                alliance_id=affiliation['alliance_id'] if 'alliance_id' in affiliation else None,
                faction_id=affiliation['faction_id'] if 'faction_id' in affiliation else None,
            )
            return model
        except (HTTPNotFound, HTTPUnprocessableEntity, ObjectNotFound):
            raise ObjectNotFound(character_id, 'character')

    def _fetch_character_name(self, character_id: int) -> str:
        """Fetch character name from ESI."""
        data = self.client.Universe.post_universe_names(ids=[character_id]).result()
        character = data.pop() if data else None
        if (
            not character
            or character["category"] != "character"
            or character["id"] != character_id
        ):
            raise ObjectNotFound(character_id, 'character')
        return character["name"]

    def get_all_factions(self):
        """Fetch all factions from ESI."""
        if not self._faction_list:
            self._faction_list = self.client.Universe.get_universe_factions().result()
        return self._faction_list

    def get_faction(self, faction_id: int):
        """Fetch faction from ESI."""
        faction_id = int(faction_id)
        try:
            if not self._faction_list:
                _ = self.get_all_factions()
            for f in self._faction_list:
                if f['faction_id'] == faction_id:
                    return Entity(id=f['faction_id'], name=f['name'])
            else:
                raise KeyError()
        except (HTTPNotFound, HTTPUnprocessableEntity, KeyError):
            raise ObjectNotFound(faction_id, 'faction')

    def get_itemtype(self, type_id: int) -> ItemType:
        """Fetch inventory item from ESI."""
        try:
            data = self.client.Universe.get_universe_types_type_id(type_id=type_id).result()
            return ItemType(id=type_id, name=data['name'])
        except (HTTPNotFound, HTTPUnprocessableEntity):
            raise ObjectNotFound(type_id, 'type')


provider = EveSwaggerProvider()



from django.template.loader import render_to_string


from typing import List, Optional


class MenuItemHook:
    """
    Auth Hook for generating Side Menu Items
    """
    def __init__(self, text: str, classes: str, url_name: str, order: Optional[int] = None, navactive: List = []):
        """
        :param text: The text shown as menu item, e.g. usually the name of the app.
        :type text: str
        :param classes: The classes that should be applied to the menu item icon
        :type classes: List[str]
        :param url_name: The name of the Django URL to use
        :type url_name: str
        :param order: An integer which specifies the order of the menu item, lowest to highest. Community apps are free to use any order above `1000`. Numbers below are served for Auth.
        :type order: Optional[int], optional
        :param navactive: A list of views or namespaces the link should be highlighted on. See [django-navhelper](https://github.com/geelweb/django-navhelper#navactive) for usage. Defaults to the supplied `url_name`.
        :type navactive: List, optional
        """

        self.text = text
        self.classes = classes
        self.url_name = url_name
        self.template = 'public/menuitem.html'
        self.order = order if order is not None else 9999

        # count is an integer shown next to the menu item as badge when count != None
        # apps need to set the count in their child class, e.g. in render() method
        self.count = None

        navactive = navactive or []
        navactive.append(url_name)
        self.navactive = navactive

    def render(self, request):
        return render_to_string(self.template,
                                {'item': self},
                                request=request)



from django.contrib import admin
from django.db import models
from .models import AutogroupsConfig, ManagedCorpGroup, ManagedAllianceGroup

import logging


logger = logging.getLogger(__name__)


def sync_user_groups(modeladmin, request, queryset):
    for agc in queryset:
        logger.debug(f"update_all_states_group_membership for {agc}")
        agc.update_all_states_group_membership()


@admin.register(AutogroupsConfig)
class AutogroupsConfigAdmin(admin.ModelAdmin):
    formfield_overrides = {
        models.CharField: {'strip': False}
    }

    def get_readonly_fields(self, request, obj=None):
        if obj:  # This is the case when obj is already created i.e. it's an edit
            return [
                'corp_group_prefix', 'corp_name_source', 'alliance_group_prefix', 'alliance_name_source',
                'replace_spaces', 'replace_spaces_with'
            ]
        else:
            return []

    def get_actions(self, request):
        actions = super().get_actions(request)
        actions['sync_user_groups'] = (sync_user_groups,
                                        'sync_user_groups',
                                        'Sync all users groups for this Autogroup Config')
        return actions


admin.site.register(ManagedCorpGroup)
admin.site.register(ManagedAllianceGroup)



from unittest.mock import patch

from amqp.exceptions import ChannelError

from django.test import TestCase

from allianceauth.authentication.core.celery_workers import (
    active_tasks_count, queued_tasks_count,
)

MODULE_PATH = "allianceauth.authentication.core.celery_workers"


@patch(MODULE_PATH + ".current_app")
class TestActiveTasksCount(TestCase):
    def test_should_return_correct_count_when_no_active_tasks(self, mock_current_app):
        # given
        mock_current_app.control.inspect.return_value.active.return_value = {
            "queue": []
        }
        # when
        result = active_tasks_count()
        # then
        self.assertEqual(result, 0)

    def test_should_return_correct_task_count_for_active_tasks(self, mock_current_app):
        # given
        mock_current_app.control.inspect.return_value.active.return_value = {
            "queue": [1, 2, 3]
        }
        # when
        result = active_tasks_count()
        # then
        self.assertEqual(result, 3)

    def test_should_return_correct_task_count_for_multiple_queues(
        self, mock_current_app
    ):
        # given
        mock_current_app.control.inspect.return_value.active.return_value = {
            "queue_1": [1, 2],
            "queue_2": [3, 4],
        }
        # when
        result = active_tasks_count()
        # then
        self.assertEqual(result, 4)

    def test_should_return_none_when_celery_not_available(self, mock_current_app):
        # given
        mock_current_app.control.inspect.return_value.active.return_value = None
        # when
        result = active_tasks_count()
        # then
        self.assertIsNone(result)


@patch(MODULE_PATH + ".current_app")
class TestQueuedTasksCount(TestCase):
    def test_should_return_queue_length_when_queue_exists(self, mock_current_app):
        # given
        mock_conn = (
            mock_current_app.connection_or_acquire.return_value.__enter__.return_value
        )
        mock_conn.default_channel.queue_declare.return_value.message_count = 7
        # when
        result = queued_tasks_count()
        # then
        self.assertEqual(result, 7)

    def test_should_return_0_when_queue_does_not_exists(self, mock_current_app):
        # given
        mock_current_app.connection_or_acquire.side_effect = ChannelError
        # when
        result = queued_tasks_count()
        # then
        self.assertEqual(result, 0)

    def test_should_return_None_on_other_errors(self, mock_current_app):
        # given
        mock_current_app.connection_or_acquire.side_effect = RuntimeError
        # when
        result = queued_tasks_count()
        # then
        self.assertIsNone(result)



# Generated by Django 1.10.1 on 2016-09-05 21:39

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EveAllianceInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('alliance_id', models.CharField(max_length=254)),
                ('alliance_name', models.CharField(max_length=254)),
                ('alliance_ticker', models.CharField(max_length=254)),
                ('executor_corp_id', models.CharField(max_length=254)),
                ('is_blue', models.BooleanField(default=False)),
                ('member_count', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='EveApiKeyPair',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_id', models.CharField(max_length=254)),
                ('api_key', models.CharField(max_length=254)),
                ('error_count', models.PositiveIntegerField(default=0)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EveCharacter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('character_id', models.CharField(max_length=254)),
                ('character_name', models.CharField(max_length=254)),
                ('corporation_id', models.CharField(max_length=254)),
                ('corporation_name', models.CharField(max_length=254)),
                ('corporation_ticker', models.CharField(max_length=254)),
                ('alliance_id', models.CharField(max_length=254)),
                ('alliance_name', models.CharField(max_length=254)),
                ('api_id', models.CharField(max_length=254)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EveCorporationInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corporation_id', models.CharField(max_length=254)),
                ('corporation_name', models.CharField(max_length=254)),
                ('corporation_ticker', models.CharField(max_length=254)),
                ('member_count', models.IntegerField()),
                ('is_blue', models.BooleanField(default=False)),
                ('alliance', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveAllianceInfo')),
            ],
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-18 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0013_fix_requestlog_date_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='requestlog',
            name='request_type',
            field=models.BooleanField(null=True),
        ),
    ]



import logging
from typing import Any

from celery import shared_task, chain
from requests.exceptions import HTTPError

from django.contrib.auth.models import User
from django.db.models.query import QuerySet

from allianceauth.services.tasks import QueueOnce

from . import __title__
from .app_settings import (
    DISCORD_TASKS_MAX_RETRIES, DISCORD_TASKS_RETRY_PAUSE, DISCORD_SYNC_NAMES
)
from .discord_client import DiscordApiBackoff
from .models import DiscordUser
from .utils import LoggerAddTag


logger = LoggerAddTag(logging.getLogger(__name__), __title__)

# task priority of bulk tasks
BULK_TASK_PRIORITY = 6


@shared_task(
    bind=True, name='discord.update_groups', base=QueueOnce, max_retries=None
)
def update_groups(self, user_pk: int, state_name: str = None) -> None:
    """Update roles on Discord for given user according to his current groups

    Params:
    - user_pk: PK of given user
    - state_name: optional state name to be used
    """
    _task_perform_user_action(self, user_pk, 'update_groups', state_name=state_name)


@shared_task(
    bind=True, name='discord.update_nickname', base=QueueOnce, max_retries=None
)
def update_nickname(self, user_pk: int, nickname: str = None) -> None:
    """Set nickname on Discord for given user to his main character name

    Params:
    - user_pk: PK of given user
    - nickname: optional nickname to be used instead of user's main
    """
    _task_perform_user_action(self, user_pk, 'update_nickname', nickname=nickname)


@shared_task(
    bind=True, name='discord.update_username', base=QueueOnce, max_retries=None
)
def update_username(self, user_pk: int) -> None:
    """Update locally stored Discord username from Discord server for given user

    Params:
    - user_pk: PK of given user
    """
    _task_perform_user_action(self, user_pk, 'update_username')


@shared_task(
    bind=True, name='discord.delete_user', base=QueueOnce, max_retries=None
)
def delete_user(self, user_pk: int, notify_user: bool = False) -> None:
    """Delete Discord user

    Params:
    - user_pk: PK of given user
    """
    _task_perform_user_action(self, user_pk, 'delete_user', notify_user=notify_user)


def _task_perform_user_action(self, user_pk: int, method: str, **kwargs) -> None:
    """perform a user related action incl. managing all exceptions"""
    logger.debug("Starting %s for user with pk %s", method, user_pk)
    user = User.objects.get(pk=user_pk)
    # logger.debug("user %s has state %s", user, user.profile.state)
    if DiscordUser.objects.user_has_account(user):
        logger.info("Running %s for user %s", method, user)
        try:
            success = getattr(user.discord, method)(**kwargs)

        except DiscordApiBackoff as bo:
            logger.info(
                "API back off for %s wth user %s due to %r, retrying in %s seconds",
                method,
                user,
                bo,
                bo.retry_after_seconds
            )
            raise self.retry(countdown=bo.retry_after_seconds)

        except AttributeError:
            raise ValueError(f'{method} not a valid method for DiscordUser')

        except (HTTPError, ConnectionError):
            logger.warning(
                '%s failed for user %s, retrying in %d secs',
                method,
                user,
                DISCORD_TASKS_RETRY_PAUSE,
                exc_info=True
            )
            if self.request.retries < DISCORD_TASKS_MAX_RETRIES:
                raise self.retry(countdown=DISCORD_TASKS_RETRY_PAUSE)
            else:
                logger.error(
                    '%s failed for user %s after max retries',
                    method,
                    user,
                    exc_info=True
                )
        except Exception:
            logger.error(
                '%s for user %s failed due to unexpected exception',
                method,
                user,
                exc_info=True
            )

        else:
            if success is None and method != 'delete_user':
                delete_user.delay(user.pk, notify_user=True)

    else:
        logger.debug(
            'User %s does not have a discord account, skipping %s', user, method
        )


@shared_task(name='discord.update_all_groups')
def update_all_groups() -> None:
    """Update roles for all known users with a Discord account."""
    discord_users_qs = DiscordUser.objects.all()
    _bulk_update_groups_for_users(discord_users_qs)


@shared_task(name='discord.update_groups_bulk')
def update_groups_bulk(user_pks: list) -> None:
    """Update roles for list of users with a Discord account in bulk."""
    discord_users_qs = DiscordUser.objects\
        .filter(user__pk__in=user_pks)\
        .select_related()
    _bulk_update_groups_for_users(discord_users_qs)


def _bulk_update_groups_for_users(discord_users_qs: QuerySet) -> None:
    logger.info(
        "Starting to bulk update discord roles for %d users", discord_users_qs.count()
    )
    update_groups_chain = list()
    for discord_user in discord_users_qs:
        update_groups_chain.append(update_groups.si(discord_user.user.pk))

    chain(update_groups_chain).apply_async(priority=BULK_TASK_PRIORITY)


@shared_task(name='discord.update_all_nicknames')
def update_all_nicknames() -> None:
    """Update nicknames for all known users with a Discord account."""
    discord_users_qs = DiscordUser.objects.all()
    _bulk_update_nicknames_for_users(discord_users_qs)


@shared_task(name='discord.update_nicknames_bulk')
def update_nicknames_bulk(user_pks: list) -> None:
    """Update nicknames for list of users with a Discord account in bulk."""
    discord_users_qs = DiscordUser.objects\
        .filter(user__pk__in=user_pks)\
        .select_related()
    _bulk_update_nicknames_for_users(discord_users_qs)


def _bulk_update_nicknames_for_users(discord_users_qs: QuerySet) -> None:
    logger.info(
        "Starting to bulk update discord nicknames for %d users",
        discord_users_qs.count()
    )
    update_nicknames_chain = list()
    for discord_user in discord_users_qs:
        update_nicknames_chain.append(update_nickname.si(discord_user.user.pk))

    chain(update_nicknames_chain).apply_async(priority=BULK_TASK_PRIORITY)


def _task_perform_users_action(self, method: str, **kwargs) -> Any:
    """Perform an action that concerns a group of users or the whole server
    and that hits the API
    """
    result = None
    try:
        result = getattr(DiscordUser.objects, method)(**kwargs)

    except AttributeError:
        raise ValueError(f'{method} not a valid method for DiscordUser.objects')

    except DiscordApiBackoff as bo:
        logger.info(
            "API back off for %s due to %r, retrying in %s seconds",
            method,
            bo,
            bo.retry_after_seconds
        )
        raise self.retry(countdown=bo.retry_after_seconds)

    except (HTTPError, ConnectionError):
        logger.warning(
            '%s failed, retrying in %d secs',
            method,
            DISCORD_TASKS_RETRY_PAUSE,
            exc_info=True
        )
        if self.request.retries < DISCORD_TASKS_MAX_RETRIES:
            raise self.retry(countdown=DISCORD_TASKS_RETRY_PAUSE)
        else:
            logger.error('%s failed after max retries', method, exc_info=True)

    except Exception:
        logger.error('%s failed due to unexpected exception', method, exc_info=True)

    return result


@shared_task(
    bind=True, name='discord.update_servername', base=QueueOnce, max_retries=None
)
def update_servername(self) -> None:
    """Updates the Discord server name"""
    _task_perform_users_action(self, method="server_name", use_cache=False)


@shared_task(name='discord.update_all_usernames')
def update_all_usernames() -> None:
    """Update all usernames for all known users with a Discord account.
    Also updates the server name
    """
    update_servername.delay()
    discord_users_qs = DiscordUser.objects.all()
    _bulk_update_usernames_for_users(discord_users_qs)


@shared_task(name='discord.update_usernames_bulk')
def update_usernames_bulk(user_pks: list) -> None:
    """Update usernames for list of users with a Discord account in bulk."""
    discord_users_qs = DiscordUser.objects\
        .filter(user__pk__in=user_pks)\
        .select_related()
    _bulk_update_usernames_for_users(discord_users_qs)


def _bulk_update_usernames_for_users(discord_users_qs: QuerySet) -> None:
    logger.info(
        "Starting to bulk update discord usernames for %d users",
        discord_users_qs.count()
    )
    update_usernames_chain = list()
    for discord_user in discord_users_qs:
        update_usernames_chain.append(update_username.si(discord_user.user.pk))

    chain(update_usernames_chain).apply_async(priority=BULK_TASK_PRIORITY)


@shared_task(name='discord.update_all')
def update_all() -> None:
    """Updates groups and nicknames (when activated) for all users."""
    discord_users_qs = DiscordUser.objects.all()
    logger.info(
        'Starting to bulk update all for %s Discord users', discord_users_qs.count()
    )
    update_all_chain = list()
    for discord_user in discord_users_qs:
        update_all_chain.append(update_groups.si(discord_user.user.pk))
        update_all_chain.append(update_username.si(discord_user.user.pk))
        if DISCORD_SYNC_NAMES:
            update_all_chain.append(update_nickname.si(discord_user.user.pk))

    chain(update_all_chain).apply_async(priority=BULK_TASK_PRIORITY)



from django.db import models
from django.utils import timezone

from allianceauth.eveonline.models import EveCharacter


class SrpFleetMain(models.Model):
    fleet_name = models.CharField(max_length=254, default="")
    fleet_doctrine = models.CharField(max_length=254, default="")
    fleet_time = models.DateTimeField()
    fleet_srp_code = models.CharField(max_length=254, default="")
    fleet_srp_status = models.CharField(max_length=254, default="")
    fleet_commander = models.ForeignKey(EveCharacter, null=True, on_delete=models.SET_NULL)
    fleet_srp_aar_link = models.CharField(max_length=254, default="")

    def __str__(self):
        return self.fleet_name

    @property
    def total_cost(self):
        return sum(int(r.srp_total_amount) for r in self.srpuserrequest_set.all())

    @property
    def pending_requests(self):
        return self.srpuserrequest_set.filter(srp_status='Pending').count()

    class Meta:
        permissions = (('access_srp', 'Can access SRP module'),)


class SrpUserRequest(models.Model):
    SRP_STATUS_CHOICES = (
        ('Pending', 'Pending'),
        ('Approved', 'Approved'),
        ('Rejected', 'Rejected'),
    )

    killboard_link = models.CharField(max_length=254, default="")
    after_action_report_link = models.CharField(max_length=254, default="")
    additional_info = models.CharField(max_length=254, default="")
    srp_status = models.CharField(max_length=8, default="Pending", choices=SRP_STATUS_CHOICES)
    srp_total_amount = models.BigIntegerField(default=0)
    character = models.ForeignKey(EveCharacter, null=True, on_delete=models.SET_NULL)
    srp_fleet_main = models.ForeignKey(SrpFleetMain, on_delete=models.CASCADE)
    kb_total_loss = models.BigIntegerField(default=0)
    srp_ship_name = models.CharField(max_length=254, default="")
    post_time = models.DateTimeField(default=timezone.now)

    def __str__(self):
        return self.character.character_name + ' SRP request for ' + self.srp_ship_name



from unittest.mock import patch, Mock

from django.test import TestCase, override_settings

from allianceauth.tests.auth_utils import AuthUtils
from ..templatetags.auth_notifications import (
    user_unread_notification_count, notifications_refresh_time
)

MODULE_PATH = 'allianceauth.notifications.templatetags.auth_notifications'

NOTIFICATIONS_REFRESH_TIME_DEFAULT = 66
MY_NOTIFICATIONS_REFRESH_TIME = 23


@patch(MODULE_PATH + '.Notification.objects.user_unread_count')
class TestUserNotificationCount(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('magic_mike')
        AuthUtils.add_main_character(
            cls.user,
            'Magic Mike',
            '1',
            corp_id='2',
            corp_name='Pole Riders',
            corp_ticker='PRIDE',
            alliance_id='3',
            alliance_name='RIDERS'
        )

    def test_return_normal(self, mock_user_unread_count):
        unread_count = 42
        mock_user_unread_count.return_value = unread_count

        result = user_unread_notification_count(self.user)
        expected = unread_count
        self.assertEqual(result, expected)
        args, kwargs = mock_user_unread_count.call_args
        self.assertEqual(args[0], self.user.pk)

    def test_return_error_if_non_user(self, mock_user_unread_count):
        unread_count = -1
        mock_user_unread_count.return_value = unread_count

        result = user_unread_notification_count('invalid')
        expected = unread_count
        self.assertEqual(result, expected)


@patch(
    MODULE_PATH + '.Notification.NOTIFICATIONS_REFRESH_TIME_DEFAULT',
    NOTIFICATIONS_REFRESH_TIME_DEFAULT
)
class TestNotificationsRefreshTime(TestCase):

    @override_settings(NOTIFICATIONS_REFRESH_TIME=MY_NOTIFICATIONS_REFRESH_TIME)
    def test_return_from_setting(self):
        result = notifications_refresh_time()
        expected = MY_NOTIFICATIONS_REFRESH_TIME
        self.assertEqual(result, expected)

    @override_settings(NOTIFICATIONS_REFRESH_TIME=0)
    def test_refresh_time_can_be_zero(self):
        result = notifications_refresh_time()
        expected = 0
        self.assertEqual(result, expected)

    @override_settings(NOTIFICATIONS_REFRESH_TIME=None)
    def test_return_default_refresh_time_if_not_exists(self):
        result = notifications_refresh_time()
        expected = NOTIFICATIONS_REFRESH_TIME_DEFAULT
        self.assertEqual(result, expected)

    @override_settings(NOTIFICATIONS_REFRESH_TIME='33')
    def test_return_default_refresh_time_if_not_int(self):
        result = notifications_refresh_time()
        expected = NOTIFICATIONS_REFRESH_TIME_DEFAULT
        self.assertEqual(result, expected)

    @override_settings(NOTIFICATIONS_REFRESH_TIME=-1)
    def test_return_default_refresh_time_if_lt_0(self):
        result = notifications_refresh_time()
        expected = NOTIFICATIONS_REFRESH_TIME_DEFAULT
        self.assertEqual(result, expected)



import logging
import datetime
from django import forms
from django.utils import timezone
from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import gettext_lazy as _

from .models import Timer, TimerType

logger = logging.getLogger(__name__)


class TimerForm(forms.ModelForm):
    class Meta:
        model = Timer
        fields = ('details', 'system', 'planet_moon', 'structure', 'timer_type', 'objective', 'important', 'corp_timer')

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        if 'instance' in kwargs and kwargs['instance'] is not None:
            # Do conversion from db datetime to days/hours/minutes
            # for appropriate fields
            current_time = timezone.now()
            td = kwargs['instance'].eve_time - current_time
            initial = kwargs.pop('initial', dict())
            if 'days_left' not in initial:
                initial.update({'days_left': td.days})
            if 'hours_left' not in initial:
                initial.update({'hours_left': td.seconds // 3600})
            if 'minutes_left' not in initial:
                initial.update({'minutes_left': td.seconds // 60 % 60})
            kwargs.update({'initial': initial})
        super().__init__(*args, **kwargs)

    structure_choices = [('POCO', 'POCO'),
                            ('I-HUB', 'I-HUB'),
                            ('TCU', 'TCU'),
                            ('POS[S]', 'POS[S]'),
                            ('POS[M]', 'POS[M]'),
                            ('POS[L]', 'POS[L]'),
                            ('Astrahus', 'Astrahus'),
                            ('Fortizar', 'Fortizar'),
                            ('Keepstar', 'Keepstar'),
                            ('Raitaru', 'Raitaru'),
                            ('Azbel', 'Azbel'),
                            ('Sotiyo', 'Sotiyo'),
                            ('Athanor', 'Athanor'),
                            ('Tatara', 'Tatara'),
                            ('Pharolux Cyno Beacon', 'Pharolux Cyno Beacon'),
                            ('Tenebrex Cyno Jammer', 'Tenebrex Cyno Jammer'),
                            ('Ansiblex Jump Gate', 'Ansiblex Jump Gate'),
                            ('Moon Mining Cycle', 'Moon Mining Cycle'),
                            (_('Other'), _('Other'))]
    objective_choices = [('Friendly', _('Friendly')),
                            ('Hostile', _('Hostile')),
                            ('Neutral', _('Neutral'))]

    details = forms.CharField(max_length=254, required=True, label=_('Details'))
    system = forms.CharField(max_length=254, required=True, label=_("System"))
    planet_moon = forms.CharField(max_length=254, label=_("Planet/Moon"), required=False, initial="")
    structure = forms.ChoiceField(choices=structure_choices, required=True, label=_("Structure Type"))
    timer_type = forms.ChoiceField(choices=TimerType.choices, label=_("Timer Type"))
    objective = forms.ChoiceField(choices=objective_choices, required=True, label=_("Objective"))
    absolute_checkbox = forms.BooleanField(label=_("Absolute Timer"), required=False, initial=False)
    absolute_time = forms.CharField(required=False,label=_("Date and Time"))
    days_left = forms.IntegerField(required=False, label=_("Days Remaining"), validators=[MinValueValidator(0)])
    hours_left = forms.IntegerField(required=False, label=_("Hours Remaining"),
                                    validators=[MinValueValidator(0), MaxValueValidator(23)])
    minutes_left = forms.IntegerField(required=False, label=_("Minutes Remaining"),
                                        validators=[MinValueValidator(0), MaxValueValidator(59)])
    important = forms.BooleanField(label=_("Important"), required=False)
    corp_timer = forms.BooleanField(label=_("Corp-Restricted"), required=False)


    def save(self, commit=True):
        timer = super().save(commit=False)

        # Get character
        character = self.user.profile.main_character
        corporation = character.corporation
        logger.debug("Determined timer save request on behalf "
                    "of character {} corporation {}".format(character, corporation))

        days_left = self.cleaned_data['days_left']
        hours_left = self.cleaned_data['hours_left']
        minutes_left = self.cleaned_data['minutes_left']
        absolute_time = self.cleaned_data['absolute_time']

        if days_left or hours_left or minutes_left:
            # Calculate future time
            future_time = datetime.timedelta(days=days_left, hours=hours_left, minutes=minutes_left)
            current_time = timezone.now()
            eve_time = current_time + future_time
            logger.debug(f"Determined timer eve time is {eve_time} - current time {current_time}, adding {future_time}")
        elif absolute_time:
            # Use absolute time
            eve_time = absolute_time
        else:
            raise ValueError("Either future time or absolute time must be provided.")

        timer.eve_time = eve_time
        timer.eve_character = character
        timer.eve_corp = corporation
        timer.user = self.user

        if commit:
            timer.save()

        return timer



# Generated by Django 3.1.2 on 2020-10-11 10:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0010_mumbleuser_certhash'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mumbleuser',
            name='pwhash',
            field=models.CharField(max_length=90),
        ),
    ]



from django.urls import path
from . import views

app_name = "groupmanagement"

urlpatterns = [
    # groups
    path("groups/", views.groups_view, name="groups"),
    path("group/request/join/<int:group_id>/", views.group_request_add, name="request_add"),
    path(
        "group/request/leave/<int:group_id>/", views.group_request_leave, name="request_leave"
    ),
    # group management
    path("groupmanagement/requests/", views.group_management, name="management"),
    path("groupmanagement/membership/", views.group_membership, name="membership"),
    path(
        "groupmanagement/membership/<int:group_id>/",
        views.group_membership_list,
        name="membership",
    ),
    path(
        "groupmanagement/membership/<int:group_id>/audit-log/",
        views.group_membership_audit,
        name="audit_log",
    ),
    path(
        "groupmanagement/membership/<int:group_id>/remove/<int:user_id>/",
        views.group_membership_remove,
        name="membership_remove",
    ),
    path(
        "groupmanagement/request/join/accept/<int:group_request_id>/",
        views.group_accept_request,
        name="accept_request",
    ),
    path(
        "groupmanagement/request/join/reject/<int:group_request_id>/",
        views.group_reject_request,
        name="reject_request",
    ),
    path(
        "groupmanagement/request/leave/accept/<int:group_request_id>/",
        views.group_leave_accept_request,
        name="leave_accept_request",
    ),
    path(
        "groupmanagement/request/leave/reject/<int:group_request_id>/",
        views.group_leave_reject_request,
        name="leave_reject_request",
    ),
]



from django.test import TestCase

from allianceauth.tests.auth_utils import AuthUtils

from ..core import NotifyApiWrapper
from ..models import Notification


class TestUserNotificationCount(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = AuthUtils.create_user("bruce_wayne")

    def test_should_add_danger_notification(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify.danger(user=self.user, title="title", message="message")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.DANGER)

    def test_should_add_info_notification(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify.info(user=self.user, title="title", message="message")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.INFO)

    def test_should_add_success_notification(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify.success(user=self.user, title="title", message="message")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.SUCCESS)

    def test_should_add_warning_notification(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify.warning(user=self.user, title="title", message="message")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.WARNING)

    def test_should_add_info_notification_via_callable(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify(user=self.user, title="title", message="message")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.INFO)

    def test_should_add_danger_notification_via_callable(self):
        # given
        notify = NotifyApiWrapper()
        # when
        notify(user=self.user, title="title", message="message", level="danger")
        # then
        obj = Notification.objects.first()
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.title, "title")
        self.assertEqual(obj.message, "message")
        self.assertEqual(obj.level, Notification.Level.DANGER)



from django.apps import AppConfig


class GroupManagementConfig(AppConfig):
    name = 'allianceauth.groupmanagement'
    label = 'groupmanagement'
    verbose_name = 'Group Management'

    def ready(self):
        from . import signals  # noqa: F401



from django.urls import include, path

from . import views

app_name = 'teamspeak3'

module_urls = [
    # Teamspeak3 service control
    path('activate/', views.activate_teamspeak3, name='activate'),
    path('deactivate/', views.deactivate_teamspeak3, name='deactivate'),
    path('reset_perm/', views.reset_teamspeak3_perm, name='reset_perm'),
    path(
        'admin_update_ts3_groups/',
        views.admin_update_ts3_groups,
        name='admin_update_ts3_groups'
    ),

    # Teamspeak Urls
    path('verify/', views.verify_teamspeak3, name='verify'),
]

urlpatterns = [
    path('teamspeak3/', include((module_urls, app_name), namespace=app_name)),
]



"""Public interface for community apps who want to interact with the Discord server
of the current Alliance Auth instance.

Example
=======

Here is an example for using the api to fetch the current roles from the configured Discord server.

.. code-block:: python

    from allianceauth.services.modules.discord.api import create_bot_client, discord_guild_id

    client = create_bot_client()  # create a new Discord client
    guild_id = discord_guild_id()  # get the ID of the configured Discord server
    roles = client.guild_roles(guild_id)  # fetch the roles from our Discord server

.. seealso::
    The docs for the client class can be found here: :py:class:`~allianceauth.services.modules.discord.discord_client.client.DiscordClient`
"""

from typing import Optional

from .app_settings import DISCORD_GUILD_ID
from .core import create_bot_client, group_to_role, server_name  # noqa
from .discord_client.models import Role  # noqa
from .models import DiscordUser  # noqa

__all__ = ["create_bot_client", "group_to_role", "server_name", "DiscordUser", "Role"]


def discord_guild_id() -> Optional[int]:
    """Guild ID of configured Discord server.

    Returns:
        Guild ID or ``None`` if not configured
    """
    return int(DISCORD_GUILD_ID) if DISCORD_GUILD_ID else None



from django import template
from django.conf import settings

from allianceauth.hooks import get_hooks

register = template.Library()


def hook_to_name(th):
    return f"{th.__class__.__module__}.{th.__class__.__name__}"


def get_theme_from_hooks(theme, hooks):
    for h in hooks:
        _h = h()
        _hn = hook_to_name(_h)
        if _hn == theme:
            return _h


def get_theme(request):
    theme = settings.DEFAULT_THEME
    hooks = get_hooks('theme_hook')

    try:
        theme = request.session.get('THEME', settings.DEFAULT_THEME_DARK if request.session.get('NIGHT_MODE', False) is True else settings.DEFAULT_THEME)
    except AttributeError:
        pass

    theme_hook = get_theme_from_hooks(theme, hooks)

    if not theme_hook:
        theme_hook = get_theme_from_hooks(settings.DEFAULT_THEME, hooks)

    return theme_hook


def get_theme_context(request):
    return {
        'theme': get_theme(request)
    }


@register.inclusion_tag('theme/theme_imports_css.html', takes_context=True)
def theme_css(context):
    request = context['request']
    return get_theme_context(request)


@register.simple_tag(takes_context=True)
def theme_html_tags(context):
    request = context['request']
    theme = get_theme(request)
    return getattr(theme, "html_tags", "")


@register.simple_tag(takes_context=True)
def header_padding_size(context):
    request = context['request']
    theme = get_theme(request)
    return getattr(theme, "header_padding")


@register.inclusion_tag('theme/theme_imports_js.html', takes_context=True)
def theme_js(context):
    request = context['request']
    return get_theme_context(request)


@register.inclusion_tag('theme/theme_select.html', takes_context=True)
def theme_select(context):
    request = context['request']
    return {
        'next': request.path,
        'themes': get_hooks('theme_hook')
    }



from allianceauth.hooks import DashboardItemHook
from allianceauth import hooks
from .views import dashboard_characters, dashboard_groups, dashboard_admin


class UserCharactersHook(DashboardItemHook):
    def __init__(self):
        DashboardItemHook.__init__(
            self,
            dashboard_characters,
            5
        )


class UserGroupsHook(DashboardItemHook):
    def __init__(self):
        DashboardItemHook.__init__(
            self,
            dashboard_groups,
            5
        )


class AdminHook(DashboardItemHook):
    def __init__(self):
        DashboardItemHook.__init__(
            self,
            dashboard_admin,
            0
        )


@hooks.register('dashboard_hook')
def register_character_hook():
    return UserCharactersHook()


@hooks.register('dashboard_hook')
def register_groups_hook():
    return UserGroupsHook()


@hooks.register('dashboard_hook')
def register_admin_hook():
    return AdminHook()



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect
from django.utils.translation import gettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm

from .manager import XenForoManager
from .models import XenforoUser
from .tasks import XenforoTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'xenforo.access_xenforo'


@login_required
@permission_required(ACCESS_PERM)
def activate_xenforo_forum(request):
    logger.debug("activate_xenforo_forum called by user %s" % request.user)
    character = request.user.profile.main_character
    logger.debug(f"Adding XenForo user for user {request.user} with main character {character}")
    result = XenForoManager.add_user(XenforoTasks.get_username(request.user), request.user.email)
    # Based on XenAPI's response codes
    if result['response']['status_code'] == 200:
        XenforoUser.objects.update_or_create(user=request.user, defaults={'username': result['username']})
        logger.info("Updated user %s with XenForo credentials. Updating groups." % request.user)
        messages.success(request, _('Activated XenForo account.'))
        credentials = {
            'username': result['username'],
            'password': result['password'],
        }
        return render(request, 'services/service_credentials.html',
                        context={'credentials': credentials, 'service': 'XenForo'})

    else:
        logger.error("Unsuccessful attempt to activate xenforo for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your XenForo account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_xenforo_forum(request):
    logger.debug("deactivate_xenforo_forum called by user %s" % request.user)
    if XenforoTasks.delete_user(request.user):
        logger.info("Successfully deactivated XenForo for user %s" % request.user)
        messages.success(request, _('Deactivated XenForo account.'))
    else:
        messages.error(request, _('An error occurred while processing your XenForo account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_xenforo_password(request):
    logger.debug("reset_xenforo_password called by user %s" % request.user)
    if XenforoTasks.has_account(request.user):
        result = XenForoManager.reset_password(request.user.xenforo.username)
        # Based on XenAPI's response codes
        if result['response']['status_code'] == 200:
            logger.info("Successfully reset XenForo password for user %s" % request.user)
            messages.success(request, _('Reset XenForo account password.'))
            credentials = {
                'username': request.user.xenforo.username,
                'password': result['password'],
            }
            return render(request, 'services/service_credentials.html',
                        context={'credentials': credentials, 'service': 'XenForo'})
    logger.error("Unsuccessful attempt to reset XenForo password for user %s" % request.user)
    messages.error(request, _('An error occurred while processing your XenForo account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_xenforo_password(request):
    logger.debug("set_xenforo_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and XenforoTasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = XenForoManager.update_user_password(request.user.xenforo.username, password)
            if result['response']['status_code'] == 200:
                logger.info("Successfully reset XenForo password for user %s" % request.user)
                messages.success(request, _('Changed XenForo password.'))
            else:
                logger.error("Failed to install custom XenForo password for user %s" % request.user)
                messages.error(request, _('An error occurred while processing your XenForo account.'))
            return redirect('services:services')
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'Forum'}
    return render(request, 'services/service_password.html', context=context)



# Generated by Django 3.1.5 on 2021-01-07 21:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0003_make_strings_more_stringy'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='notification',
            options={},
        ),
        migrations.AlterField(
            model_name='notification',
            name='timestamp',
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='viewed',
            field=models.BooleanField(db_index=True, default=False),
        ),
    ]



import requests_mock

from django.test.utils import override_settings

from allianceauth.analytics.tasks import (
    analytics_event,
    send_ga_tracking_celery_event)
from allianceauth.utils.testing import NoSocketsTestCase


GOOGLE_ANALYTICS_DEBUG_URL = 'https://www.google-analytics.com/debug/mp/collect'


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@requests_mock.Mocker()
class TestAnalyticsTasks(NoSocketsTestCase):
    def test_analytics_event(self, requests_mocker):
        requests_mocker.register_uri('POST', GOOGLE_ANALYTICS_DEBUG_URL)
        analytics_event(
            namespace='allianceauth.analytics',
            task='send_tests',
            label='test',
            value=1,
            result="Success",
            event_type='Stats')

    def test_send_ga_tracking_celery_event_sent(self, requests_mocker):
        # given
        requests_mocker.register_uri('POST', GOOGLE_ANALYTICS_DEBUG_URL)
        token = 'G-6LYSMYK8DE'
        secret = 'KLlpjLZ-SRGozS5f5wb_kw',
        category = 'test'
        action = 'test'
        label = 'test'
        value = '1'
        # when
        task = send_ga_tracking_celery_event(
            token,
            secret,
            category,
            action,
            label,
            value)
        # then
        self.assertEqual(task, 200)

    def test_send_ga_tracking_celery_event_success(self, requests_mocker):
        # given
        requests_mocker.register_uri(
            'POST',
            GOOGLE_ANALYTICS_DEBUG_URL,
            json={"validationMessages": []}
        )
        token = 'G-6LYSMYK8DE'
        secret = 'KLlpjLZ-SRGozS5f5wb_kw',
        category = 'test'
        action = 'test'
        label = 'test'
        value = '1'
        # when
        task = send_ga_tracking_celery_event(
            token,
            secret,
            category,
            action,
            label,
            value)
        # then
        self.assertTrue(task, 200)



from typing import List, Optional


class ThemeHook:
    """
        Theme hook for injecting a Bootstrap 5 Theme and associated JS into alliance auth.
        these can be local or CDN delivered
    """

    def __init__(self,
                name: str,
                description: str,
                css: List[dict],
                js: List[dict],
                css_template: Optional[str] = None,
                js_template: Optional[str] = None,
                html_tags: Optional[str] = "",
                header_padding: Optional[str] = "4em"):
        """
        :param name: Theme python name
        :type name: str
        :param description: Theme verbose name
        :type description: str
        :param css: CSS paths to load
        :type css: List[dict]
        :param js: JS paths to load
        :type js: List[dict]
        :param css_template: _description_, defaults to None
        :type css_template: Optional[str], optional
        :param js_template: _description_, defaults to None
        :type js_template: Optional[str], optional
        """
        self.name = name
        self.description = description

        # Direct from CDN
        self.css = css
        self.js = js

        # Load a django template with static file definitions
        self.css_template = css_template
        self.js_template = js_template

        self.html_tags = html_tags
        self.header_padding = header_padding
    def get_name(self):
        return f"{self.__class__.__module__}.{self.__class__.__name__}"



from django import forms
from django.utils.translation import gettext_lazy as _

from .manager import Teamspeak3Manager


class TeamspeakJoinForm(forms.Form):
    username = forms.CharField(widget=forms.HiddenInput())

    def clean(self):
        with Teamspeak3Manager() as ts3man:
            if ts3man._get_userid(self.cleaned_data['username']):
                return self.cleaned_data
        raise forms.ValidationError(_("Unable to locate user %s on server") % self.cleaned_data['username'])



# Generated by Django 3.1.1 on 2021-01-05 14:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0012_index_additions'),
    ]

    operations = [
        migrations.AddField(
            model_name='evecorporationinfo',
            name='ceo_id',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
    ]



import inspect
import json
import os
from unittest.mock import patch

from django.contrib.auth.models import User
from django.utils.timezone import now
from django.test import TestCase

from allianceauth.tests.auth_utils import AuthUtils

from ..managers import SRPManager
from ..models import SrpUserRequest, SrpFleetMain

MODULE_PATH = 'allianceauth.srp.managers'

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(
    inspect.currentframe()
)))


def load_data(filename):
    """loads given JSON file from `testdata` sub folder and returns content"""
    with open(
        currentdir + '/testdata/%s.json' % filename, encoding='utf-8'
    ) as f:
        data = json.load(f)

    return data


class TestSrpManager(TestCase):

    def test_can_extract_kill_id(self):
        link = 'https://zkillboard.com/kill/81973979/'
        expected = 81973979
        self.assertEqual(int(SRPManager.get_kill_id(link)), expected)

    @patch(MODULE_PATH + '.esi')
    @patch(MODULE_PATH + '.requests.get')
    def test_can_get_kill_data(self, mock_get, mock_provider):
        mock_get.return_value.json.return_value = load_data(
            'zkillboard_killmail_api_81973979'
        )
        mock_provider.client.Killmails.\
            get_killmails_killmail_id_killmail_hash.return_value.\
            result.return_value = load_data(
                'get_killmails_killmail_id_killmail_hash_81973979'
            )

        ship_type, ship_value, victim_id = SRPManager.get_kill_data(81973979)
        self.assertEqual(ship_type, 19720)
        self.assertEqual(ship_value, 3177859026.86)
        self.assertEqual(victim_id, 93330670)

    @patch(MODULE_PATH + '.requests.get')
    def test_invalid_id_for_zkb_raises_exception(self, mock_get):
        mock_get.return_value.json.return_value = ['']

        with self.assertRaises(ValueError):
            SRPManager.get_kill_data(81973979)

    @patch(MODULE_PATH + '.esi')
    @patch(MODULE_PATH + '.requests.get')
    def test_invalid_id_for_esi_raises_exception(
        self, mock_get, mock_provider
    ):
        mock_get.return_value.json.return_value = load_data(
            'zkillboard_killmail_api_81973979'
        )
        mock_provider.client.Killmails.\
            get_killmails_killmail_id_killmail_hash.return_value.\
            result.return_value = None

        with self.assertRaises(ValueError):
            SRPManager.get_kill_data(81973979)

    def test_pending_requests_count_for_user(self):
        user = AuthUtils.create_member("Bruce Wayne")

        # when no permission to approve SRP requests
        # then return None
        self.assertIsNone(SRPManager.pending_requests_count_for_user(user))

        # given permission to approve SRP requests
        # when no open requests
        # then return 0
        AuthUtils.add_permission_to_user_by_name("auth.srp_management", user)
        user = User.objects.get(pk=user.pk)
        self.assertEqual(SRPManager.pending_requests_count_for_user(user), 0)

        # given permission to approve SRP requests
        # when 1 pending request
        # then return 1
        fleet = SrpFleetMain.objects.create(fleet_time=now())
        SrpUserRequest.objects.create(
            killboard_link="https://zkillboard.com/kill/79111612/",
            srp_status="Pending",
            srp_fleet_main=fleet,
        )
        SrpUserRequest.objects.create(
            killboard_link="https://zkillboard.com/kill/79111612/",
            srp_status="Approved",
            srp_fleet_main=fleet,
        )
        self.assertEqual(SRPManager.pending_requests_count_for_user(user), 1)



from django.urls import path

from . import views

app_name = 'timerboard'

urlpatterns = [
    path('', views.TimerView.as_view(), name='view'),
    path('add/', views.AddTimerView.as_view(), name='add'),
    path('remove/<int:pk>/', views.RemoveTimerView.as_view(), name='delete'),
    path('edit/<int:pk>/', views.EditTimerView.as_view(), name='edit'),
]



# Generated by Django 1.10 on 2017-10-20 13:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0002_choices_for_questions'),
    ]

    operations = [
        migrations.AddField(
            model_name='applicationquestion',
            name='multi_select',
            field=models.BooleanField(default=False),
        ),
    ]



from django.urls import include, path

from . import views

app_name = 'openfire'

module_urls = [
    # Jabber Service Control
    path('activate/', views.activate_jabber, name='activate'),
    path('deactivate/', views.deactivate_jabber, name='deactivate'),
    path('reset_password/', views.reset_jabber_password, name='reset_password'),
    path('set_password/', views.set_jabber_password, name='set_password'),
    path('broadcast/', views.jabber_broadcast_view, name='broadcast'),
]

urlpatterns = [
    path('openfire/', include((module_urls, app_name), namespace=app_name)),
]
